.device ATtiny13
.include "tn13def.inc"
.list
;--   
.def   w1=r16       ;    
.def   w2=r17       ;   -  
.def   Flags=r18    ; 
.def   TIN_H=r19    ;    ,  
.def   TIN_L=r20    ;    ,  
.def   IIN_H=r21    ;     ,  
.def   IIN_L=r22    ;     ,  

.def   TOUT_H=r23   ;     ,  
.def   TOUT_L=r24   ;     ,  
.def   IOUT_H=r25   ;      ,  
.def   IOUT_L=r26   ;      ,  

.def   OC_H=r27      ;    
.def   OC_L=r28      ;    

.equ   TINCF=0      ;   16-     
.equ   IINCEN=1     ;     (0 -   )
.equ   STARTF=2     ;   
.equ   TOprosa=127  ; 9,6 /128 = 75 
;--------------------------
.equ   PWM_IN=1           ;  
.equ   PWM_OUT=0          ;  

;--  
; SPL -   
; ACSR -  
; DDRB -   
; PORTB -  
; TCCR0B -  
; TIMSK0 -   
; TCNT0 -  
; OCR0A -   
; TIFR0 -    
; PB0 -  , PB1 - 
;--   
        .cseg
        .org 0
        rjmp Init  ;    
;--  
        reti                ; INT0
        reti                ; Pin Change
        rjmp TimerOverflow  ; Timer
        reti                ; EEPROM
        reti                ; comparator
        reti                ; timer compare match A
        reti                ; timer compare match B
        reti                ; watchdog
        reti                ; ADC
;--  
Init: ldi w1,RAMEND     ;   
      out SPL,w1        ;     RAM
      sbi ACSR,7        ;  
      ldi w1,1<<PWM_OUT ;      (1-, 0-)
      out DDRB,w1
      clr w1            ;   , 
      out PORTB,w1      ;   = 0
;--  
      ldi w1,(1<<WGM00)|(1<<WGM01)
      out TCCR0A,w1     ;  OCR0A, OCR0B , Fast PWM
      ldi w1,TOprosa    ;  ,     
      out OCR0A,w1
      ldi w1, 1<<TOIE0  ;   
	  out TIMSK0,w1
;---    
	  sbr Flags,1<<IINCEN ;  
;-- =0  ,  (IOUT_H OR IOUT_L)=0
;-- =100  ,  (IOUT_H OR IOUT_L)  0,  (TOUT_H OR TOUT_L)=0
;--  =0
;--    
      ldi w1,0b00001001 ;  , Fast PWM,  
      out TCCR0B,w1
      sei
;--  
Start: rjmp Start

;----------------------------------------------------------------------
;----------------------------------------------------------------------
;--- Interrupt ---
TimerOverflow:
;--          
      in  w2,PINB          ;   
      rcall IncCounterTIN  ;     
      clr w1
	  bld w1,PWM_IN        ;   T    W   PWM_IN
      bst w2,PWM_IN        ;      T
	  andi w2,1<<PWM_IN    ;   w2  ,   
	  eor w1,w2            ;   ?
      breq NoChanged       ; ,   
;---
Changed:
      brts  FromLowToHi     ;   T=1 -      
;--    
FromHiToLow:                ;  
      sbrc  Flags, TINCF    ;    -  
      rjmp  Changed_HL_OVF
Changed_HL_No_OVF:
      rcall IncCounterIIN   ;    
      sbr  Flags,1<<IINCEN  ;    
      rjmp  Update_Output
Changed_HL_OVF:
      ser IOUT_L
	  clr TOUT_H
	  clr TOUT_L            ; =100%
      rcall ClrCounters     ;  
	  clr   Flags           ;  
	  rjmp  Update_Output
;--    
FromLowToHi:                ;  ,   
      sbrc  Flags, TINCF    ;    -  
      rjmp  Changed_LH_OVF
Changed_LH_No_OVF:
      sbrc  Flags, STARTF   ;  ,    STARTF
      rcall CalcToutIout
Changed_LH_No_Calc:
      clr   Flags
	  sbr   Flags, 1<<STARTF   ;  
	  rcall ClrCounters
	  rjmp  Update_Output
Changed_LH_OVF:
      clr   IOUT_H          ; =0
	  clr   IOUT_L
      rjmp  Changed_LH_No_Calc
;--  
NoChanged:
      sbrc  Flags, TINCF    ;    -  
      rjmp  NoChanged_OVF
NoChanged_No_OVF:
      sbrs  Flags,IINCEN    ;  ,   
	  rcall IncCounterIIN
      rjmp  Update_Output
NoChanged_OVF:
      clr IOUT_L
	  clr IOUT_H
	  clr TOUT_H
	  clr TOUT_L            ; =0%
	  brtc Next1            ;    -  =0%
	  ser IOUT_L            ; =100%
Next1:
      rcall ClrCounters
	  clr   Flags           ;  

;--  
Update_Output:
      mov  w1, IOUT_L
      or   w1, IOUT_H       ; IOUT_H v IOUT_L
	  brne NotNull          ;    0%
      cbi  PORTB,PWM_OUT    ; clear PWM_OUT to 0
	  reti
NotNull:
      mov  w1, TOUT_H
      or   w1, TOUT_L       ; TOUT_H v TOUT_L
	  brne NotFull          ;    100% 
      sbi  PORTB,PWM_OUT    ; set PWM_OUT to 1
      reti
NotFull:
      rcall IncCounterOC
      cp    OC_L, TOUT_L
	  brne  Next2
	  cp    OC_H, TOUT_H
	  brne  Next2
      clr   OC_L
	  clr   OC_H
Next2:
      cp   OC_H, IOUT_H     ; OC_H - TOUT_H
	  brlo Lower            ; jump if OC_H lower IOUT_H
	  brne HierOrEqual      ; jump if OC_H  bigger IOUT_H
      cp   OC_L, IOUT_L     ; compare if OC_H equal IOUT_H
	  brlo Lower
HierOrEqual:
      cbi  PORTB,PWM_OUT    ; clear PWM_OUT to 0
      reti
Lower:
      sbi  PORTB,PWM_OUT    ; set PWM_OUT to 1
      reti

;-----------------------
;---  --------
IncCounterTIN:
      inc   TIN_L
	  brne  No_Z1
      inc   TIN_H
	  brne  No_Z1
	  sbr   Flags, 1<<TINCF  ; set TINCF
No_Z1:
      ret

IncCounterIIN:
      inc   IIN_L
	  brne  No_Z2
      inc   IIN_H
No_Z2:
      ret

IncCounterOC:
      inc   OC_L
	  brne  No_Z3
      inc   OC_H
No_Z3:
      ret

CalcToutIout:
      mov  TOUT_H,TIN_H     ; copy TIN_H->T_OUTH, TIN_L->T_OUTL
	  mov  TOUT_L,TIN_L
      clc                   ; clear carry flag
      ror  TOUT_H           ;     2
	  ror  TOUT_L
	  mov  IOUT_H,IIN_H     ; copy IIN_H->IOUT_H, IIN_L->IOUT_L
	  mov  IOUT_L,IIN_L
	  clc
	  ror  IOUT_H           ;     2
	  ror  IOUT_L
      ret

ClrCounters:
      clr TIN_L
	  clr TIN_H
	  clr IIN_L
	  clr IIN_H
	  ser OC_H            ;  OC = -1
	  ser OC_L            ; (set OC to -1)
      ret
;-----------------------
